from enum import _EnumDict
from enum import Enum
from enum import EnumMeta


class StrEnumMeta(EnumMeta):
    def __new__(metacls, cls, bases, oldclassdict):
        """
        Scan through `oldclassdict` and convert any value that is a plain tuple
        into a `str` of the name instead
        """
        newclassdict = _EnumDict()
        setattr(newclassdict, "_cls_name", cls)
        for k, v in oldclassdict.items():
            if v == ():
                v = k
            newclassdict[k] = v
        return super().__new__(metacls, cls, bases, newclassdict)


class SPIRVEnum(str, Enum, metaclass=StrEnumMeta):
    def __str__(self) -> str:
        # Workaround since SPIR-V enums have "None" as values and it's reserved in Python
        return "None" if self.name == "NONE" else self.name

    def __repr__(self) -> str:
        return str(self)


class SourceLanguage(SPIRVEnum):
    Unknown = ()
    ESSL = ()
    GLSL = ()
    OpenCL_C = ()
    OpenCL_CPP = ()
    HLSL = ()
    CPP_for_OpenCL = ()


class ExecutionModel(SPIRVEnum):
    Vertex = ()
    TessellationControl = ()
    TessellationEvaluation = ()
    Geometry = ()
    Fragment = ()
    GLCompute = ()
    Kernel = ()
    TaskNV = ()
    MeshNV = ()
    RayGenerationKHR = ()
    RayGenerationNV = ()
    IntersectionKHR = ()
    IntersectionNV = ()
    AnyHitKHR = ()
    AnyHitNV = ()
    ClosestHitKHR = ()
    ClosestHitNV = ()
    MissKHR = ()
    MissNV = ()
    CallableKHR = ()
    CallableNV = ()


class AddressingModel(SPIRVEnum):
    Logical = ()
    Physical32 = ()
    Physical64 = ()
    PhysicalStorageBuffer64 = ()
    PhysicalStorageBuffer64EXT = ()


class MemoryModel(SPIRVEnum):
    Simple = ()
    GLSL450 = ()
    OpenCL = ()
    Vulkan = ()
    VulkanKHR = ()


class ExecutionMode(SPIRVEnum):
    Invocations = ()
    SpacingEqual = ()
    SpacingFractionalEven = ()
    SpacingFractionalOdd = ()
    VertexOrderCw = ()
    VertexOrderCcw = ()
    PixelCenterInteger = ()
    OriginUpperLeft = ()
    OriginLowerLeft = ()
    EarlyFragmentTests = ()
    PointMode = ()
    Xfb = ()
    DepthReplacing = ()
    DepthGreater = ()
    DepthLess = ()
    DepthUnchanged = ()
    LocalSize = ()
    LocalSizeHint = ()
    InputPoints = ()
    InputLines = ()
    InputLinesAdjacency = ()
    Triangles = ()
    InputTrianglesAdjacency = ()
    Quads = ()
    Isolines = ()
    OutputVertices = ()
    OutputPoints = ()
    OutputLineStrip = ()
    OutputTriangleStrip = ()
    VecTypeHint = ()
    ContractionOff = ()
    Initializer = ()
    Finalizer = ()
    SubgroupSize = ()
    SubgroupsPerWorkgroup = ()
    SubgroupsPerWorkgroupId = ()
    LocalSizeId = ()
    LocalSizeHintId = ()
    SubgroupUniformControlFlowKHR = ()
    PostDepthCoverage = ()
    DenormPreserve = ()
    DenormFlushToZero = ()
    SignedZeroInfNanPreserve = ()
    RoundingModeRTE = ()
    RoundingModeRTZ = ()
    StencilRefReplacingEXT = ()
    OutputLinesNV = ()
    OutputPrimitivesNV = ()
    DerivativeGroupQuadsNV = ()
    DerivativeGroupLinearNV = ()
    OutputTrianglesNV = ()
    PixelInterlockOrderedEXT = ()
    PixelInterlockUnorderedEXT = ()
    SampleInterlockOrderedEXT = ()
    SampleInterlockUnorderedEXT = ()
    ShadingRateInterlockOrderedEXT = ()
    ShadingRateInterlockUnorderedEXT = ()
    SharedLocalMemorySizeINTEL = ()
    RoundingModeRTPINTEL = ()
    RoundingModeRTNINTEL = ()
    FloatingPointModeALTINTEL = ()
    FloatingPointModeIEEEINTEL = ()
    MaxWorkgroupSizeINTEL = ()
    MaxWorkDimINTEL = ()
    NoGlobalOffsetINTEL = ()
    NumSIMDWorkitemsINTEL = ()
    SchedulerTargetFmaxMhzINTEL = ()


class StorageClass(SPIRVEnum):
    UniformConstant = ()
    Input = ()
    Uniform = ()
    Output = ()
    Workgroup = ()
    CrossWorkgroup = ()
    Private = ()
    Function = ()
    Generic = ()
    PushConstant = ()
    AtomicCounter = ()
    Image = ()
    StorageBuffer = ()
    CallableDataKHR = ()
    CallableDataNV = ()
    IncomingCallableDataKHR = ()
    IncomingCallableDataNV = ()
    RayPayloadKHR = ()
    RayPayloadNV = ()
    HitAttributeKHR = ()
    HitAttributeNV = ()
    IncomingRayPayloadKHR = ()
    IncomingRayPayloadNV = ()
    ShaderRecordBufferKHR = ()
    ShaderRecordBufferNV = ()
    PhysicalStorageBuffer = ()
    PhysicalStorageBufferEXT = ()
    CodeSectionINTEL = ()
    DeviceOnlyINTEL = ()
    HostOnlyINTEL = ()


class Dim(SPIRVEnum):
    Dim1D = ()
    Dim2D = ()
    Dim3D = ()
    Cube = ()
    Rect = ()
    Buffer = ()
    SubpassData = ()


class SamplerAddressingMode(SPIRVEnum):
    NONE = ()
    ClampToEdge = ()
    Clamp = ()
    Repeat = ()
    RepeatMirrored = ()


class SamplerFilterMode(SPIRVEnum):
    Nearest = ()
    Linear = ()


class ImageFormat(SPIRVEnum):
    Unknown = ()
    Rgba32f = ()
    Rgba16f = ()
    R32f = ()
    Rgba8 = ()
    Rgba8Snorm = ()
    Rg32f = ()
    Rg16f = ()
    R11fG11fB10f = ()
    R16f = ()
    Rgba16 = ()
    Rgb10A2 = ()
    Rg16 = ()
    Rg8 = ()
    R16 = ()
    R8 = ()
    Rgba16Snorm = ()
    Rg16Snorm = ()
    Rg8Snorm = ()
    R16Snorm = ()
    R8Snorm = ()
    Rgba32i = ()
    Rgba16i = ()
    Rgba8i = ()
    R32i = ()
    Rg32i = ()
    Rg16i = ()
    Rg8i = ()
    R16i = ()
    R8i = ()
    Rgba32ui = ()
    Rgba16ui = ()
    Rgba8ui = ()
    R32ui = ()
    Rgb10a2ui = ()
    Rg32ui = ()
    Rg16ui = ()
    Rg8ui = ()
    R16ui = ()
    R8ui = ()
    R64ui = ()
    R64i = ()


class ImageChannelOrder(SPIRVEnum):
    R = ()
    A = ()
    RG = ()
    RA = ()
    RGB = ()
    RGBA = ()
    BGRA = ()
    ARGB = ()
    Intensity = ()
    Luminance = ()
    Rx = ()
    RGx = ()
    RGBx = ()
    Depth = ()
    DepthStencil = ()
    sRGB = ()
    sRGBx = ()
    sRGBA = ()
    sBGRA = ()
    ABGR = ()


class ImageChannelDataType(SPIRVEnum):
    SnormInt8 = ()
    SnormInt16 = ()
    UnormInt8 = ()
    UnormInt16 = ()
    UnormShort565 = ()
    UnormShort555 = ()
    UnormInt101010 = ()
    SignedInt8 = ()
    SignedInt16 = ()
    SignedInt32 = ()
    UnsignedInt8 = ()
    UnsignedInt16 = ()
    UnsignedInt32 = ()
    HalfFloat = ()
    Float = ()
    UnormInt24 = ()
    UnormInt101010_2 = ()


class ImageOperandsShift(SPIRVEnum):
    Bias = ()
    Lod = ()
    Grad = ()
    ConstOffset = ()
    Offset = ()
    ConstOffsets = ()
    Sample = ()
    MinLod = ()
    MakeTexelAvailable = ()
    MakeTexelAvailableKHR = ()
    MakeTexelVisible = ()
    MakeTexelVisibleKHR = ()
    NonPrivateTexel = ()
    NonPrivateTexelKHR = ()
    VolatileTexel = ()
    VolatileTexelKHR = ()
    SignExtend = ()
    ZeroExtend = ()
    Nontemporal = ()
    Offsets = ()


class ImageOperandsMask(SPIRVEnum):
    MaskNone = ()
    Bias = ()
    Lod = ()
    Grad = ()
    ConstOffset = ()
    Offset = ()
    ConstOffsets = ()
    Sample = ()
    MinLod = ()
    MakeTexelAvailable = ()
    MakeTexelAvailableKHR = ()
    MakeTexelVisible = ()
    MakeTexelVisibleKHR = ()
    NonPrivateTexel = ()
    NonPrivateTexelKHR = ()
    VolatileTexel = ()
    VolatileTexelKHR = ()
    SignExtend = ()
    ZeroExtend = ()
    Nontemporal = ()
    Offsets = ()


class FPFastMathModeShift(SPIRVEnum):
    NotNaN = ()
    NotInf = ()
    NSZ = ()
    AllowRecip = ()
    Fast = ()
    AllowContractFastINTEL = ()
    AllowReassocINTEL = ()


class FPFastMathModeMask(SPIRVEnum):
    MaskNone = ()
    NotNaN = ()
    NotInf = ()
    NSZ = ()
    AllowRecip = ()
    Fast = ()
    AllowContractFastINTEL = ()
    AllowReassocINTEL = ()


class FPRoundingMode(SPIRVEnum):
    RTE = ()
    RTZ = ()
    RTP = ()
    RTN = ()


class LinkageType(SPIRVEnum):
    Export = ()
    Import = ()
    LinkOnceODR = ()


class AccessQualifier(SPIRVEnum):
    ReadOnly = ()
    WriteOnly = ()
    ReadWrite = ()


class FunctionParameterAttribute(SPIRVEnum):
    Zext = ()
    Sext = ()
    ByVal = ()
    Sret = ()
    NoAlias = ()
    NoCapture = ()
    NoWrite = ()
    NoReadWrite = ()


class Decoration(SPIRVEnum):
    RelaxedPrecision = ()
    SpecId = ()
    Block = ()
    BufferBlock = ()
    RowMajor = ()
    ColMajor = ()
    ArrayStride = ()
    MatrixStride = ()
    GLSLShared = ()
    GLSLPacked = ()
    CPacked = ()
    BuiltIn = ()
    NoPerspective = ()
    Flat = ()
    Patch = ()
    Centroid = ()
    Sample = ()
    Invariant = ()
    Restrict = ()
    Aliased = ()
    Volatile = ()
    Constant = ()
    Coherent = ()
    NonWritable = ()
    NonReadable = ()
    Uniform = ()
    UniformId = ()
    SaturatedConversion = ()
    Stream = ()
    Location = ()
    Component = ()
    Index = ()
    Binding = ()
    DescriptorSet = ()
    Offset = ()
    XfbBuffer = ()
    XfbStride = ()
    FuncParamAttr = ()
    FPRoundingMode = ()
    FPFastMathMode = ()
    LinkageAttributes = ()
    NoContraction = ()
    InputAttachmentIndex = ()
    Alignment = ()
    MaxByteOffset = ()
    AlignmentId = ()
    MaxByteOffsetId = ()
    NoSignedWrap = ()
    NoUnsignedWrap = ()
    ExplicitInterpAMD = ()
    OverrideCoverageNV = ()
    PassthroughNV = ()
    ViewportRelativeNV = ()
    SecondaryViewportRelativeNV = ()
    PerPrimitiveNV = ()
    PerViewNV = ()
    PerTaskNV = ()
    PerVertexKHR = ()
    PerVertexNV = ()
    NonUniform = ()
    NonUniformEXT = ()
    RestrictPointer = ()
    RestrictPointerEXT = ()
    AliasedPointer = ()
    AliasedPointerEXT = ()
    BindlessSamplerNV = ()
    BindlessImageNV = ()
    BoundSamplerNV = ()
    BoundImageNV = ()
    SIMTCallINTEL = ()
    ReferencedIndirectlyINTEL = ()
    ClobberINTEL = ()
    SideEffectsINTEL = ()
    VectorComputeVariableINTEL = ()
    FuncParamIOKindINTEL = ()
    VectorComputeFunctionINTEL = ()
    StackCallINTEL = ()
    GlobalVariableOffsetINTEL = ()
    CounterBuffer = ()
    HlslCounterBufferGOOGLE = ()
    HlslSemanticGOOGLE = ()
    UserSemantic = ()
    UserTypeGOOGLE = ()
    FunctionRoundingModeINTEL = ()
    FunctionDenormModeINTEL = ()
    RegisterINTEL = ()
    MemoryINTEL = ()
    NumbanksINTEL = ()
    BankwidthINTEL = ()
    MaxPrivateCopiesINTEL = ()
    SinglepumpINTEL = ()
    DoublepumpINTEL = ()
    MaxReplicatesINTEL = ()
    SimpleDualPortINTEL = ()
    MergeINTEL = ()
    BankBitsINTEL = ()
    ForcePow2DepthINTEL = ()
    BurstCoalesceINTEL = ()
    CacheSizeINTEL = ()
    DontStaticallyCoalesceINTEL = ()
    PrefetchINTEL = ()
    StallEnableINTEL = ()
    FuseLoopsInFunctionINTEL = ()
    BufferLocationINTEL = ()
    IOPipeStorageINTEL = ()
    FunctionFloatingPointModeINTEL = ()
    SingleElementVectorINTEL = ()
    VectorComputeCallableFunctionINTEL = ()
    MediaBlockIOINTEL = ()


class BuiltIn(SPIRVEnum):
    Position = ()
    PointSize = ()
    ClipDistance = ()
    CullDistance = ()
    VertexId = ()
    InstanceId = ()
    PrimitiveId = ()
    InvocationId = ()
    Layer = ()
    ViewportIndex = ()
    TessLevelOuter = ()
    TessLevelInner = ()
    TessCoord = ()
    PatchVertices = ()
    FragCoord = ()
    PointCoord = ()
    FrontFacing = ()
    SampleId = ()
    SamplePosition = ()
    SampleMask = ()
    FragDepth = ()
    HelperInvocation = ()
    NumWorkgroups = ()
    WorkgroupSize = ()
    WorkgroupId = ()
    LocalInvocationId = ()
    GlobalInvocationId = ()
    LocalInvocationIndex = ()
    WorkDim = ()
    GlobalSize = ()
    EnqueuedWorkgroupSize = ()
    GlobalOffset = ()
    GlobalLinearId = ()
    SubgroupSize = ()
    SubgroupMaxSize = ()
    NumSubgroups = ()
    NumEnqueuedSubgroups = ()
    SubgroupId = ()
    SubgroupLocalInvocationId = ()
    VertexIndex = ()
    InstanceIndex = ()
    SubgroupEqMask = ()
    SubgroupEqMaskKHR = ()
    SubgroupGeMask = ()
    SubgroupGeMaskKHR = ()
    SubgroupGtMask = ()
    SubgroupGtMaskKHR = ()
    SubgroupLeMask = ()
    SubgroupLeMaskKHR = ()
    SubgroupLtMask = ()
    SubgroupLtMaskKHR = ()
    BaseVertex = ()
    BaseInstance = ()
    DrawIndex = ()
    PrimitiveShadingRateKHR = ()
    DeviceIndex = ()
    ViewIndex = ()
    ShadingRateKHR = ()
    BaryCoordNoPerspAMD = ()
    BaryCoordNoPerspCentroidAMD = ()
    BaryCoordNoPerspSampleAMD = ()
    BaryCoordSmoothAMD = ()
    BaryCoordSmoothCentroidAMD = ()
    BaryCoordSmoothSampleAMD = ()
    BaryCoordPullModelAMD = ()
    FragStencilRefEXT = ()
    ViewportMaskNV = ()
    SecondaryPositionNV = ()
    SecondaryViewportMaskNV = ()
    PositionPerViewNV = ()
    ViewportMaskPerViewNV = ()
    FullyCoveredEXT = ()
    TaskCountNV = ()
    PrimitiveCountNV = ()
    PrimitiveIndicesNV = ()
    ClipDistancePerViewNV = ()
    CullDistancePerViewNV = ()
    LayerPerViewNV = ()
    MeshViewCountNV = ()
    MeshViewIndicesNV = ()
    BaryCoordKHR = ()
    BaryCoordNV = ()
    BaryCoordNoPerspKHR = ()
    BaryCoordNoPerspNV = ()
    FragSizeEXT = ()
    FragmentSizeNV = ()
    FragInvocationCountEXT = ()
    InvocationsPerPixelNV = ()
    LaunchIdKHR = ()
    LaunchIdNV = ()
    LaunchSizeKHR = ()
    LaunchSizeNV = ()
    WorldRayOriginKHR = ()
    WorldRayOriginNV = ()
    WorldRayDirectionKHR = ()
    WorldRayDirectionNV = ()
    ObjectRayOriginKHR = ()
    ObjectRayOriginNV = ()
    ObjectRayDirectionKHR = ()
    ObjectRayDirectionNV = ()
    RayTminKHR = ()
    RayTminNV = ()
    RayTmaxKHR = ()
    RayTmaxNV = ()
    InstanceCustomIndexKHR = ()
    InstanceCustomIndexNV = ()
    ObjectToWorldKHR = ()
    ObjectToWorldNV = ()
    WorldToObjectKHR = ()
    WorldToObjectNV = ()
    HitTNV = ()
    HitKindKHR = ()
    HitKindNV = ()
    CurrentRayTimeNV = ()
    IncomingRayFlagsKHR = ()
    IncomingRayFlagsNV = ()
    RayGeometryIndexKHR = ()
    WarpsPerSMNV = ()
    SMCountNV = ()
    WarpIDNV = ()
    SMIDNV = ()


class SelectionControlShift(SPIRVEnum):
    Flatten = ()
    DontFlatten = ()


class SelectionControlMask(SPIRVEnum):
    NONE = ()
    Flatten = ()
    DontFlatten = ()


class LoopControlShift(SPIRVEnum):
    Unroll = ()
    DontUnroll = ()
    DependencyInfinite = ()
    DependencyLength = ()
    MinIterations = ()
    MaxIterations = ()
    IterationMultiple = ()
    PeelCount = ()
    PartialCount = ()
    InitiationIntervalINTEL = ()
    MaxConcurrencyINTEL = ()
    DependencyArrayINTEL = ()
    PipelineEnableINTEL = ()
    LoopCoalesceINTEL = ()
    MaxInterleavingINTEL = ()
    SpeculatedIterationsINTEL = ()
    NoFusionINTEL = ()


class LoopControlMask(SPIRVEnum):
    MaskNone = ()
    Unroll = ()
    DontUnroll = ()
    DependencyInfinite = ()
    DependencyLength = ()
    MinIterations = ()
    MaxIterations = ()
    IterationMultiple = ()
    PeelCount = ()
    PartialCount = ()
    InitiationIntervalINTEL = ()
    MaxConcurrencyINTEL = ()
    DependencyArrayINTEL = ()
    PipelineEnableINTEL = ()
    LoopCoalesceINTEL = ()
    MaxInterleavingINTEL = ()
    SpeculatedIterationsINTEL = ()
    NoFusionINTEL = ()


class FunctionControlShift(SPIRVEnum):
    Inline = ()
    DontInline = ()
    Pure = ()
    Const = ()
    OptNoneINTEL = ()


class FunctionControlMask(SPIRVEnum):
    NONE = ()
    Inline = ()
    DontInline = ()
    Pure = ()
    Const = ()
    OptNoneINTEL = ()


class MemorySemanticsShift(SPIRVEnum):
    Acquire = ()
    Release = ()
    AcquireRelease = ()
    SequentiallyConsistent = ()
    UniformMemory = ()
    SubgroupMemory = ()
    WorkgroupMemory = ()
    CrossWorkgroupMemory = ()
    AtomicCounterMemory = ()
    ImageMemory = ()
    OutputMemory = ()
    OutputMemoryKHR = ()
    MakeAvailable = ()
    MakeAvailableKHR = ()
    MakeVisible = ()
    MakeVisibleKHR = ()
    Volatile = ()


class MemorySemanticsMask(SPIRVEnum):
    MaskNone = ()
    Acquire = ()
    Release = ()
    AcquireRelease = ()
    SequentiallyConsistent = ()
    UniformMemory = ()
    SubgroupMemory = ()
    WorkgroupMemory = ()
    CrossWorkgroupMemory = ()
    AtomicCounterMemory = ()
    ImageMemory = ()
    OutputMemory = ()
    OutputMemoryKHR = ()
    MakeAvailable = ()
    MakeAvailableKHR = ()
    MakeVisible = ()
    MakeVisibleKHR = ()
    Volatile = ()


class MemoryAccessShift(SPIRVEnum):
    Volatile = ()
    Aligned = ()
    Nontemporal = ()
    MakePointerAvailable = ()
    MakePointerAvailableKHR = ()
    MakePointerVisible = ()
    MakePointerVisibleKHR = ()
    NonPrivatePointer = ()
    NonPrivatePointerKHR = ()


class MemoryAccessMask(SPIRVEnum):
    MaskNone = ()
    Volatile = ()
    Aligned = ()
    Nontemporal = ()
    MakePointerAvailable = ()
    MakePointerAvailableKHR = ()
    MakePointerVisible = ()
    MakePointerVisibleKHR = ()
    NonPrivatePointer = ()
    NonPrivatePointerKHR = ()


class Scope(SPIRVEnum):
    CrossDevice = ()
    Device = ()
    Workgroup = ()
    Subgroup = ()
    Invocation = ()
    QueueFamily = ()
    QueueFamilyKHR = ()
    ShaderCallKHR = ()


class GroupOperation(SPIRVEnum):
    Reduce = ()
    InclusiveScan = ()
    ExclusiveScan = ()
    ClusteredReduce = ()
    PartitionedReduceNV = ()
    PartitionedInclusiveScanNV = ()
    PartitionedExclusiveScanNV = ()


class KernelEnqueueFlags(SPIRVEnum):
    NoWait = ()
    WaitKernel = ()
    WaitWorkGroup = ()


class KernelProfilingInfoShift(SPIRVEnum):
    CmdExecTime = ()


class KernelProfilingInfoMask(SPIRVEnum):
    MaskNone = ()
    CmdExecTime = ()


class Capability(SPIRVEnum):
    Matrix = ()
    Shader = ()
    Geometry = ()
    Tessellation = ()
    Addresses = ()
    Linkage = ()
    Kernel = ()
    Vector16 = ()
    Float16Buffer = ()
    Float16 = ()
    Float64 = ()
    Int64 = ()
    Int64Atomics = ()
    ImageBasic = ()
    ImageReadWrite = ()
    ImageMipmap = ()
    Pipes = ()
    Groups = ()
    DeviceEnqueue = ()
    LiteralSampler = ()
    AtomicStorage = ()
    Int16 = ()
    TessellationPointSize = ()
    GeometryPointSize = ()
    ImageGatherExtended = ()
    StorageImageMultisample = ()
    UniformBufferArrayDynamicIndexing = ()
    SampledImageArrayDynamicIndexing = ()
    StorageBufferArrayDynamicIndexing = ()
    StorageImageArrayDynamicIndexing = ()
    ClipDistance = ()
    CullDistance = ()
    ImageCubeArray = ()
    SampleRateShading = ()
    ImageRect = ()
    SampledRect = ()
    GenericPointer = ()
    Int8 = ()
    InputAttachment = ()
    SparseResidency = ()
    MinLod = ()
    Sampled1D = ()
    Image1D = ()
    SampledCubeArray = ()
    SampledBuffer = ()
    ImageBuffer = ()
    ImageMSArray = ()
    StorageImageExtendedFormats = ()
    ImageQuery = ()
    DerivativeControl = ()
    InterpolationFunction = ()
    TransformFeedback = ()
    GeometryStreams = ()
    StorageImageReadWithoutFormat = ()
    StorageImageWriteWithoutFormat = ()
    MultiViewport = ()
    SubgroupDispatch = ()
    NamedBarrier = ()
    PipeStorage = ()
    GroupNonUniform = ()
    GroupNonUniformVote = ()
    GroupNonUniformArithmetic = ()
    GroupNonUniformBallot = ()
    GroupNonUniformShuffle = ()
    GroupNonUniformShuffleRelative = ()
    GroupNonUniformClustered = ()
    GroupNonUniformQuad = ()
    ShaderLayer = ()
    ShaderViewportIndex = ()
    UniformDecoration = ()
    FragmentShadingRateKHR = ()
    SubgroupBallotKHR = ()
    DrawParameters = ()
    WorkgroupMemoryExplicitLayoutKHR = ()
    WorkgroupMemoryExplicitLayout8BitAccessKHR = ()
    WorkgroupMemoryExplicitLayout16BitAccessKHR = ()
    SubgroupVoteKHR = ()
    StorageBuffer16BitAccess = ()
    StorageUniformBufferBlock16 = ()
    StorageUniform16 = ()
    UniformAndStorageBuffer16BitAccess = ()
    StoragePushConstant16 = ()
    StorageInputOutput16 = ()
    DeviceGroup = ()
    MultiView = ()
    VariablePointersStorageBuffer = ()
    VariablePointers = ()
    AtomicStorageOps = ()
    SampleMaskPostDepthCoverage = ()
    StorageBuffer8BitAccess = ()
    UniformAndStorageBuffer8BitAccess = ()
    StoragePushConstant8 = ()
    DenormPreserve = ()
    DenormFlushToZero = ()
    SignedZeroInfNanPreserve = ()
    RoundingModeRTE = ()
    RoundingModeRTZ = ()
    RayQueryProvisionalKHR = ()
    RayQueryKHR = ()
    RayTraversalPrimitiveCullingKHR = ()
    RayTracingKHR = ()
    Float16ImageAMD = ()
    ImageGatherBiasLodAMD = ()
    FragmentMaskAMD = ()
    StencilExportEXT = ()
    ImageReadWriteLodAMD = ()
    Int64ImageEXT = ()
    ShaderClockKHR = ()
    SampleMaskOverrideCoverageNV = ()
    GeometryShaderPassthroughNV = ()
    ShaderViewportIndexLayerEXT = ()
    ShaderViewportIndexLayerNV = ()
    ShaderViewportMaskNV = ()
    ShaderStereoViewNV = ()
    PerViewAttributesNV = ()
    FragmentFullyCoveredEXT = ()
    MeshShadingNV = ()
    ImageFootprintNV = ()
    FragmentBarycentricKHR = ()
    FragmentBarycentricNV = ()
    ComputeDerivativeGroupQuadsNV = ()
    FragmentDensityEXT = ()
    ShadingRateNV = ()
    GroupNonUniformPartitionedNV = ()
    ShaderNonUniform = ()
    ShaderNonUniformEXT = ()
    RuntimeDescriptorArray = ()
    RuntimeDescriptorArrayEXT = ()
    InputAttachmentArrayDynamicIndexing = ()
    InputAttachmentArrayDynamicIndexingEXT = ()
    UniformTexelBufferArrayDynamicIndexing = ()
    UniformTexelBufferArrayDynamicIndexingEXT = ()
    StorageTexelBufferArrayDynamicIndexing = ()
    StorageTexelBufferArrayDynamicIndexingEXT = ()
    UniformBufferArrayNonUniformIndexing = ()
    UniformBufferArrayNonUniformIndexingEXT = ()
    SampledImageArrayNonUniformIndexing = ()
    SampledImageArrayNonUniformIndexingEXT = ()
    StorageBufferArrayNonUniformIndexing = ()
    StorageBufferArrayNonUniformIndexingEXT = ()
    StorageImageArrayNonUniformIndexing = ()
    StorageImageArrayNonUniformIndexingEXT = ()
    InputAttachmentArrayNonUniformIndexing = ()
    InputAttachmentArrayNonUniformIndexingEXT = ()
    UniformTexelBufferArrayNonUniformIndexing = ()
    UniformTexelBufferArrayNonUniformIndexingEXT = ()
    StorageTexelBufferArrayNonUniformIndexing = ()
    StorageTexelBufferArrayNonUniformIndexingEXT = ()
    RayTracingNV = ()
    RayTracingMotionBlurNV = ()
    VulkanMemoryModel = ()
    VulkanMemoryModelKHR = ()
    VulkanMemoryModelDeviceScope = ()
    VulkanMemoryModelDeviceScopeKHR = ()
    PhysicalStorageBufferAddresses = ()
    PhysicalStorageBufferAddressesEXT = ()
    ComputeDerivativeGroupLinearNV = ()
    RayTracingProvisionalKHR = ()
    CooperativeMatrixNV = ()
    FragmentShaderSampleInterlockEXT = ()
    FragmentShaderShadingRateInterlockEXT = ()
    ShaderSMBuiltinsNV = ()
    FragmentShaderPixelInterlockEXT = ()
    DemoteToHelperInvocation = ()
    DemoteToHelperInvocationEXT = ()
    BindlessTextureNV = ()
    SubgroupShuffleINTEL = ()
    SubgroupBufferBlockIOINTEL = ()
    SubgroupImageBlockIOINTEL = ()
    SubgroupImageMediaBlockIOINTEL = ()
    RoundToInfinityINTEL = ()
    FloatingPointModeINTEL = ()
    IntegerFunctions2INTEL = ()
    FunctionPointersINTEL = ()
    IndirectReferencesINTEL = ()
    AsmINTEL = ()
    AtomicFloat32MinMaxEXT = ()
    AtomicFloat64MinMaxEXT = ()
    AtomicFloat16MinMaxEXT = ()
    VectorComputeINTEL = ()
    VectorAnyINTEL = ()
    ExpectAssumeKHR = ()
    SubgroupAvcMotionEstimationINTEL = ()
    SubgroupAvcMotionEstimationIntraINTEL = ()
    SubgroupAvcMotionEstimationChromaINTEL = ()
    VariableLengthArrayINTEL = ()
    FunctionFloatControlINTEL = ()
    FPGAMemoryAttributesINTEL = ()
    FPFastMathModeINTEL = ()
    ArbitraryPrecisionIntegersINTEL = ()
    ArbitraryPrecisionFloatingPointINTEL = ()
    UnstructuredLoopControlsINTEL = ()
    FPGALoopControlsINTEL = ()
    KernelAttributesINTEL = ()
    FPGAKernelAttributesINTEL = ()
    FPGAMemoryAccessesINTEL = ()
    FPGAClusterAttributesINTEL = ()
    LoopFuseINTEL = ()
    FPGABufferLocationINTEL = ()
    ArbitraryPrecisionFixedPointINTEL = ()
    USMStorageClassesINTEL = ()
    IOPipesINTEL = ()
    BlockingPipesINTEL = ()
    FPGARegINTEL = ()
    DotProductInputAll = ()
    DotProductInputAllKHR = ()
    DotProductInput4x8Bit = ()
    DotProductInput4x8BitKHR = ()
    DotProductInput4x8BitPacked = ()
    DotProductInput4x8BitPackedKHR = ()
    DotProduct = ()
    DotProductKHR = ()
    BitInstructions = ()
    AtomicFloat32AddEXT = ()
    AtomicFloat64AddEXT = ()
    LongConstantCompositeINTEL = ()
    OptNoneINTEL = ()
    AtomicFloat16AddEXT = ()
    DebugInfoModuleINTEL = ()


class RayFlagsShift(SPIRVEnum):
    OpaqueKHR = ()
    NoOpaqueKHR = ()
    TerminateOnFirstHitKHR = ()
    SkipClosestHitShaderKHR = ()
    CullBackFacingTrianglesKHR = ()
    CullFrontFacingTrianglesKHR = ()
    CullOpaqueKHR = ()
    CullNoOpaqueKHR = ()
    SkipTrianglesKHR = ()
    SkipAABBsKHR = ()


class RayFlagsMask(SPIRVEnum):
    MaskNone = ()
    OpaqueKHR = ()
    NoOpaqueKHR = ()
    TerminateOnFirstHitKHR = ()
    SkipClosestHitShaderKHR = ()
    CullBackFacingTrianglesKHR = ()
    CullFrontFacingTrianglesKHR = ()
    CullOpaqueKHR = ()
    CullNoOpaqueKHR = ()
    SkipTrianglesKHR = ()
    SkipAABBsKHR = ()


class RayQueryIntersection(SPIRVEnum):
    RayQueryCandidateIntersectionKHR = ()
    RayQueryCommittedIntersectionKHR = ()


class RayQueryCommittedIntersectionType(SPIRVEnum):
    RayQueryCommittedIntersectionNoneKHR = ()
    RayQueryCommittedIntersectionTriangleKHR = ()
    RayQueryCommittedIntersectionGeneratedKHR = ()


class RayQueryCandidateIntersectionType(SPIRVEnum):
    RayQueryCandidateIntersectionTriangleKHR = ()
    RayQueryCandidateIntersectionAABBKHR = ()


class FragmentShadingRateShift(SPIRVEnum):
    Vertical2Pixels = ()
    Vertical4Pixels = ()
    Horizontal2Pixels = ()
    Horizontal4Pixels = ()


class FragmentShadingRateMask(SPIRVEnum):
    MaskNone = ()
    Vertical2Pixels = ()
    Vertical4Pixels = ()
    Horizontal2Pixels = ()
    Horizontal4Pixels = ()


class FPDenormMode(SPIRVEnum):
    Preserve = ()
    FlushToZero = ()


class FPOperationMode(SPIRVEnum):
    IEEE = ()
    ALT = ()


class QuantizationModes(SPIRVEnum):
    TRN = ()
    TRN_ZERO = ()
    RND = ()
    RND_ZERO = ()
    RND_INF = ()
    RND_MIN_INF = ()
    RND_CONV = ()
    RND_CONV_ODD = ()


class OverflowModes(SPIRVEnum):
    WRAP = ()
    SAT = ()
    SAT_ZERO = ()
    SAT_SYM = ()


class PackedVectorFormat(SPIRVEnum):
    PackedVectorFormat4x8Bit = ()
    PackedVectorFormat4x8BitKHR = ()
